package fr.mastah;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.regex.Pattern;

/**
 * @author Mastah
 */
public class ExtractJSDoc3 {

	/** Pattern of JSDoc3. */
	private Pattern pattern;

	/**
	 * Constructor.
	 */
	public ExtractJSDoc3() {
		this.pattern = Pattern.compile("jsdoc3.*");
	}

	/**
	 * Extract JSDoc3 to destination path.
	 * 
	 * @param theDestination destination where JSDoc3 will be extracted.
	 * @throws IOException input/output exceptions.
	 */
	public final void extract(final String theDestination) throws IOException {

		// Retrieve current jar path on file system
		String jarFilePath = this.resolveJarPath();

		// Create File from path
		File localJarFile = new File(jarFilePath);

		// Create JarFile from File
		JarFile jarFile = new JarFile(localJarFile);

		// Retrieve all entries from the jar file
		Enumeration<JarEntry> entries = jarFile.entries();

		// Current JarEntry
		JarEntry file;

		// Current file name
		String fileName;

		// Current File from the jar
		File currentFile;

		while (entries.hasMoreElements()) {

			file = entries.nextElement();

			fileName = file.getName();

			// If the file correspond to JSDoc3
			if (this.pattern.matcher(fileName).matches()) {

				// We create the destination File that should correspond to the current one
				currentFile = new java.io.File(theDestination + java.io.File.separator + fileName);

				// If the file is a directory
				if (file.isDirectory()) {
					currentFile.mkdir();
				} else {

					// The file is not a directory
					// We retrieve the stream from this file
					InputStream is = jarFile.getInputStream(file);

					// We create an out stream on the destination file
					FileOutputStream fos = new java.io.FileOutputStream(currentFile);

					// We write the content of the current file to the destination file
					while (is.available() > 0) {
						fos.write(is.read());
					}

					// We close destination and input stream
					fos.close();
					is.close();
				}
			}
		}

		// We close the jar file
		jarFile.close();

	}

	/**
	 * Resolve the path of the current Jar.
	 * 
	 * @return path of the jar file.
	 */
	private String resolveJarPath() {
		return this.getClass().getProtectionDomain().getCodeSource().getLocation().getPath();
	}

}
