package fr.mastah.mojo.process;

import java.io.IOException;
import java.io.InputStream;

import org.apache.maven.plugin.logging.Log;

/**
 * @author mastah
 */
public abstract class AbstractInputStreamHandler extends Thread {

	/** Stream being read. */
	private InputStream mStream;

	/** The StringBuffer holding the captured output. */
	private StringBuffer mCaptureBuffer;

	/** Logger. */
	private Log logger;

	/**
	 * Constructor.
	 * 
	 * @param theStream stream associated.
	 * @param theLogger logger associated.
	 */
	public AbstractInputStreamHandler(final InputStream theStream, final Log theLogger) {
		this.mStream = theStream;
		this.mCaptureBuffer = new StringBuffer();
		this.logger = theLogger;
		this.start();
	}

	/**
	 * Stream the data.
	 */
	@Override
	public final void run() {
		try {
			boolean hit = false;
			int nextChar;
			while ((nextChar = this.mStream.read()) != -1) {
				this.mCaptureBuffer.append((char) nextChar);
				hit = true;
			}
			if (hit) {
				this.doLog(this.mCaptureBuffer);
			}
		} catch (IOException ioe) {
			this.logger.error(ioe);
		}
	}

	/**
	 * Define how to log.
	 * 
	 * @param theBuffer buffer used to store incoming data from the process.
	 */
	public abstract void doLog(final StringBuffer theBuffer);

	/**
	 * Getter for logger.
	 * 
	 * @return the logger.
	 */
	public final Log getLogger() {
		return this.logger;
	}

}
